# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018 . All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************


from pyspark.sql import SparkSession

class DatasetReader:
    """
    Load distributed dataset from file.
    """
    def __init__(self):
        """
        Initialize DatasetReader

        """
        ss = SparkSession.builder.getOrCreate()
        # Create an instance of the DatasetReader using the JVM from the SparkSession 
        self._DatasetReader = ss._jvm.com.ibm.snap.ml.DatasetReader(ss._jsparkSession)

    def setFormat(self,format):
        """
        Specify the dataformat of the file.  Format values: "snap" or "libsvm" or "csv"

        :param format: data format
        :type format: string
        """

        if format not in set(['snap','libsvm','csv']):
            raise ValueError("Invalid format value. Allowed format values: 'snap', 'libsvm' or 'csv'")
        self._DatasetReader = self._DatasetReader.setFormat(format)
        return self

    def takeRange(self,idx_start,idx_end):
        """
        If not the whole dataset should be loaded specify start and end index.

        :param idx_start: first sample to load
        :type idx_start: int
        :param idx_end: last sample to load
        :type idx_end: int
        """
        self._DatasetReader = self._DatasetReader.takeRange(idx_start,idx_end)
        return self

    def setNumFt(self, x):
        """
        Set the number of features

        :param x: index
        :type x: int
        """
        self._DatasetReader = self._DatasetReader.setMaxInd(x)
        return self

    def load(self, file):
        """
        Load training data in memory

        :param file: filename
        :type file: string
        """
        return self._DatasetReader.load(file)
