# Loaders examples

In this example we generate some random dataset in order to demonstrate how the `snap_ml_spark` loaders can be used. In addition, we perform some performance comparison
on the dataset loading times.

## Generate data

We generate some random symmetric data of 10000x10000 dimensions with 0.1 density (highly sparse data).
To generate the datasets, run the following commands:
```bash
mkdir data/
python $CONDA_PREFIX/snap-ml-spark/examples/example-load_data/gen_and_store_data.py
```  

After the execution of the python script above, we will see the following data in the `data/` directory.
```
total 3.5G
  39M  data_l2.snap
  77M  data_sparse.snap.t
  77M  data_sparse.snap
 382M  data_dense.snap.t
 382M  data_dense.snap
 2.4G  data.csv
 236M  data.libsvm
```

## Dataset formats

After generating our symmetric dataset, let's see what are the different data formats provided by the `snap_ml_spark.DatasetReader`.

### snap format
The first and most important dataset format for the `snap_ml_spark` library is the `snap` format. We recommend using the snap format for two main reasons.
Firstly, as you will see in the example in the end, the snap DatasetReader is the most efficient one since the data are stored in binary format (require less memory to be stored on disk) and can be rapidly loaded (sequentially) into the memory. Secondly, snap is the only format that can store a dataset in its transpose format. This allows the users to also train their models (Ridge Regression, Logistic Regression) in their primal formulation. Being able to train a model in both its primal and dual formulation is a big advantage when dealing with highly asymmetrical  datasets by choosing the most appropriate form which minimizes the communication cost. It also provides the user with strong theoretical  guarantees (stopping criterion, optimal solution guarrantee).

One additional thing to keep in mind is that while it is required to read the dataset in its transpose format in order to train a model in its primal formulation, the primal-inference requires always the non-transposed dataset. For example, to train a primal and a dual model and perform inference in both of the two models, one needs to do the following steps:
```
# primal case
- Read transposed training dataset
- Read non-transposed test dataset
- Train primal model on the transposed training dataset
- Perform inference on the non-transposed test dataset

# dual case
- Read non-transposed training dataset
- Read non-transposed test dataset
- Train primal model on the non-transposed training dataset
- Perform inference on the non-transposed test dataset
```

The supported snap formats are the following:

#### sparse snap format
This format represents sparse datasets into binary-snap format. For example, one can convert a `libsvm dataset` to a `sparse_snap` dataset.

#### dense snap format
This format represents dense datasets into binary-snap format. For example, one can convert a dense `csv dataset` to a `dense_snap` dataset.

#### l2 snap format
This format (`l2snap`) is a specific case of the snap format where the values of a dataset can be normalized and implied during run-time.

### libsvm and csv formats
The `snap_ml_spark.DatasetReader` supports also the `libsvm` and `csv` formats for compatibility reasons. Please note that the user cannot train a model in its primal form when using these dataset formats.

## Test Loaders

One can run the entire example by executing the following commands:
```bash
spark-submit --master spark://node1.ibm.com:7077 --jars $CONDA_PREFIX/snap-ml-spark/lib/snap-ml-spark-v*-ppc64le.jar $CONDA_PREFIX/snap-ml-spark/examples/example-load_data/example-load.py --data_path path/to/data

Note: By default the location of --data_path is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

Command to run example:

spark-submit --master spark://node1.ibm.com:7077 --jars $CONDA_PREFIX/snap-ml-spark/lib/snap-ml-spark-v*-ppc64le.jar $CONDA_PREFIX/snap-ml-spark/examples/example-load_data/example-load.py

Command to run example with specified number of partitions:

spark-submit --master spark://node1.ibm.com:7077 --jars $CONDA_PREFIX/snap-ml-spark/lib/snap-ml-spark-v*-ppc64le.jar $CONDA_PREFIX/snap-ml-spark/examples/example-load_data/example-load.py --num_partitions 8

Note: Default value of num_partitions is 2. num_partitions can be 8, if user wants to use 8 processes.


```

One should see the the following output on a P9 machine:
```
data_l2.snap :          0.415858983994
data_sparse.snap.t :    0.53568983078
data_dense.snap :       0.565274953842
data_sparse.snap :      0.639019012451
data_dense.snap.t :     0.677714824677
data.libsvm :           3.50010991096
data.csv :              18.5543661118
```

As to be expected, the `snap` loader performs faster than the `libsvm` and the `csv` loaders.

&copy; Copyright IBM Corporation 2018, 2019
