# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from __future__ import print_function
from snap_ml_spark import DatasetReader
import argparse
import os
## SPARK

from pyspark import SparkConf, SparkContext
from pyspark.sql.session import SparkSession

defaultPath = os.getcwd()
CLI=argparse.ArgumentParser()
CLI.add_argument(
   "--data_path",
   type=str,
   default=defaultPath
)
CLI.add_argument(
   "--num_partitions",
   type=int,
   default= 1
)
args = CLI.parse_args()
# Set your own master (e.g 'spark://zhcc011.zurich.ibm.com:7077') or use 'local'
# Configure Spark (2 executors, local standalone mode)
# If one wants to run spark on a cluster, use your own spark configuration (set master , appName, etc...) 
conf = SparkConf()\
.set("spark.default.parallelism",args.num_partitions)\
.set("spark.cores.max",args.num_partitions)\
.setAppName("LoadersExamples")\
.set("spark.executor.extraJavaOptions","-XX:MaxDirectMemorySize=8192m")\
.set("spark.driver.maxResultSize",8192)
# Create spark config
sc = SparkContext(conf=conf)
# Configure Spark
spark = SparkSession(sc)

# disable spark INFO logging - comment out to enable
log4j = sc._jvm.org.apache.log4j
log4j.LogManager.getRootLogger().setLevel(log4j.Level.ERROR)

# Please use the absolute path of the file if you wish to run the example on a distributed mode
path = args.data_path + '/data/data'

import time
times = {}

print('Reading data.csv file')
t_init = time.time()
dataset = DatasetReader().setFormat("csv").load(path + ".csv")
times['data.csv'] = time.time() - t_init
print('data.csv read-time: ', times['data.csv'])

print('Reading data.libsvm file')
t_init = time.time()
dataset = DatasetReader().setFormat("libsvm").load(path + ".libsvm")
times['data.libsvm'] = time.time() - t_init
print('data.libsvm read-time: ', times['data.libsvm'])

print('Reading data_sparse.snap file')
t_init = time.time()
dataset = DatasetReader().setFormat("snap").load(path + "_sparse.snap")
times['data_sparse.snap'] = time.time() - t_init
print('data_sparse.snap read-time: ', times['data_sparse.snap'])

print('Reading data_sparse.snap.t file')
t_init = time.time()
dataset = DatasetReader().setFormat("snap").load(path + "_sparse.snap.t")
times['data_sparse.snap.t'] = time.time() - t_init
print('data_sparse.snap.t read-time: ', times['data_sparse.snap.t'])

print('Reading data_dense.snap file')
t_init = time.time()
dataset = DatasetReader().setFormat("snap").load(path + "_dense.snap")
times['data_dense.snap'] = time.time() - t_init
print('data_desne.snap read-time: ', times['data_dense.snap'])

print('Reading data_dense.snap.t file')
t_init = time.time()
dataset = DatasetReader().setFormat("snap").load(path + "_dense.snap.t")
times['data_dense.snap.t'] = time.time() - t_init
print('data_desne.snap.t read-time: ', times['data_dense.snap.t'])

print('Reading data_l2.snap file')
t_init = time.time()
dataset = DatasetReader().setFormat("snap").load(path + "_l2.snap")
times['data_l2.snap'] = time.time() - t_init
print('data_l2.snap read-time: ', times['data_l2.snap'])

print('')
for f in sorted(times, key=times.get, reverse=False):
  print(f,':\t',times[f])
