# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from __future__ import print_function
from pyspark import SparkConf, SparkContext
from snap_ml_spark import DatasetReader
from snap_ml_spark import SupportVectorMachine 
from snap_ml_spark.Metrics import accuracy

import argparse
import os
import time


defaultPath = os.getcwd()
CLI=argparse.ArgumentParser()
CLI.add_argument(
    "--use_gpu",
    action = 'store_true',
    default=False
)
CLI.add_argument(
   "--data_path",
   type=str,
   default=defaultPath
)
CLI.add_argument(
   "--num_partitions",
   type=int,
   default= 2
)
CLI.add_argument(
   "--use_balanced_class_weights",
   action = 'store_true',
   default=False
)
args = CLI.parse_args()
if args.use_balanced_class_weights:
  class_weights = "balanced"
else:
  class_weights = None

# Set your own master (e.g 'spark://zhcc011.zurich.ibm.com:7077') or use 'local'
# Configure Spark.
# If one wants to run spark on a cluster, use your own spark configuration (set master , appName, etc...) 
conf = SparkConf()\
.set("spark.default.parallelism",args.num_partitions)\
.set("spark.cores.max",args.num_partitions)\
.setAppName("SupportVectorMachineExample")\

# Create spark config
sc = SparkContext(conf=conf)

# disable spark INFO logging - comment out to enable
log4j = sc._jvm.org.apache.log4j
log4j.LogManager.getRootLogger().setLevel(log4j.Level.ERROR)

# Load training data (from csv format)
# Please use the absolute path of the file if you wish to run the example on a distributed mode
trainData = DatasetReader().setFormat("csv").load(args.data_path + "/data/small.csv")

# Load test data (from csv format)
# Please use the absolute path of the file if you wish to run the example on a distributed mode
testData = DatasetReader().setFormat("csv").load(args.data_path + "/data/small.csv")

# Create support vector machine
svm = SupportVectorMachine(use_gpu=args.use_gpu, verbose=True, max_iter=10,
                           class_weights=class_weights)

# Fit svm model on training data
snapml_t0 = time.time()
svm.fit(trainData)
snapml_time = time.time() - snapml_t0

# Perform inference on test data
pred = svm.predict(testData)

# Compute accuracy
acc = accuracy(pred)

# Print off result
print("accuracy = %.3f" % (acc))
print("Training Time=%.3f" % (snapml_time))

